import { Command } from 'commander';

declare function buildPostCommands(program: Command): void;

declare function buildWpCommands(program: Command): void;

type Wordpress = {
    domain: string;
    username: string;
    password: string;
};

declare function initStore(): void;
declare function getAllWordpress(): Promise<Wordpress[]>;
declare function addWordpress(wp: Wordpress): Promise<void>;
declare function getWordpress(domain: string): Promise<Wordpress | undefined>;
declare function removeWordpress(domain: string): Promise<Boolean>;
declare function exportWordpressList(exportFile: string): Promise<void>;
declare function importWordpressList(importFile: string): Promise<void>;

type TotalTokens = {
    promptTokens: number;
    completionTokens: number;
    total: number;
};
type PostPrompt = {
    topic: string;
    country?: string;
    intent?: string;
    audience?: string;
    language?: string;
    tone?: 'informative' | 'captivating';
    withConclusion?: boolean;
    model: 'gpt-4' | 'gpt-3.5-turbo';
    maxModelTokens?: 4000 | 8000;
    temperature?: number;
    frequencyPenalty?: number;
    presencePenalty?: number;
    logitBias?: number;
    debug?: boolean;
    debugapi?: boolean;
    apiKey?: string;
    templateFile?: string;
    templateContent?: string;
    prompts?: string[];
};
type Heading = {
    title: string;
    keywords?: string[];
    headings?: Heading[];
};
type PostOutline = {
    title: string;
    headings: Heading[];
    slug: string;
    seoTitle: string;
    seoDescription: string;
};
type Post = {
    title: string;
    content: string;
    seoTitle: string;
    seoDescription: string;
    slug: string;
    categories?: number[];
    status?: string;
    totalTokens: TotalTokens;
};
type SeoInfo = {
    slug: string;
    seoTitle: string;
    seoDescription: string;
};

/**
 * Interface for the helper class for generating a post. it defines how to generate a post
 * Each helper class must implement this interface
 * @interface
 */
interface GeneratorHelperInterface {
    init(): Promise<void>;
    isCustom(): boolean;
    generateContentOutline(): Promise<PostOutline>;
    generateMainKeyword(): Promise<string[]>;
    generateIntroduction(): Promise<string>;
    generateConclusion(): Promise<string>;
    generateHeadingContents(tableOfContent: PostOutline): Promise<string>;
    generateCustomPrompt(prompt: string): Promise<string>;
    generateSeoInfo(): Promise<SeoInfo>;
    getTotalTokens(): TotalTokens;
    getPrompt(): PostPrompt;
}

/**
 * Class for generating a post. It need a helper class to generate the post
 * Each helper class must implement the GeneratorHelperInterface
 */
declare class PostGenerator {
    private helper;
    constructor(helper: GeneratorHelperInterface);
    generate(): Promise<Post>;
    private customGenerate;
    private autoGenerate;
}
/**
 * Class for generating a post using the OpenAI API
 */
declare class OpenAIPostGenerator extends PostGenerator {
    constructor(postPrompt: PostPrompt);
}

export { Heading, OpenAIPostGenerator, Post, PostGenerator, PostOutline, PostPrompt, SeoInfo, TotalTokens, addWordpress, buildPostCommands, buildWpCommands, exportWordpressList, getAllWordpress, getWordpress, importWordpressList, initStore, removeWordpress };
