// src/bin/command/post.ts
import fs from "fs";
import { marked } from "marked";

// src/bin/question/questions.ts
import inquirer from "inquirer";
import inquirerPrompt from "inquirer-autocomplete-prompt";
import inquirerFileTreeSelection from "inquirer-file-tree-selection-prompt";
inquirer.registerPrompt("autocomplete", inquirerPrompt);
inquirer.registerPrompt("file-tree-selection", inquirerFileTreeSelection);
var LANGUAGES = [
  "english",
  "french",
  "spanish",
  "german",
  "italian",
  "russian",
  "portuguese",
  "polish",
  "turkish",
  "swedish",
  "norwegian",
  "dutch",
  "danish",
  "czech",
  "greek",
  "hungarian",
  "finnish",
  "romanian",
  "bulgarian",
  "serbian",
  "slovak",
  "croatian",
  "ukrainian",
  "slovene",
  "estonian",
  "latvian",
  "lithuanian",
  "chinese",
  "hindi",
  "arabic",
  "japanese"
];
var CONTENT_TONE = ["informative", "captivating"];
var MODELS = ["gpt-4", "gpt-3.5-turbo"];
var questions = [
  {
    type: "autocomplete",
    name: "language",
    message: "Language ?",
    source: (answersSoFar, input) => LANGUAGES.filter((language) => language.startsWith(input)),
    default: "english"
  },
  {
    type: "list",
    name: "model",
    message: "AI model ?",
    choices: MODELS,
    default: "gpt-4"
  },
  {
    type: "input",
    name: "filename",
    message: "Filename (without extension) ?",
    default: "post"
  },
  {
    type: "input",
    name: "topic",
    message: "Topic/ Artitle Title ? "
  },
  {
    type: "input",
    name: "country",
    message: "Country ?",
    default: "none"
  },
  {
    type: "input",
    name: "intent",
    message: "Intent ?",
    default: "The article should be informative and offer advice to the reader."
  },
  {
    type: "input",
    name: "audience",
    message: "Audience ?",
    default: "The article should be written for a general audience."
  },
  {
    type: "confirm",
    name: "withConclusion",
    message: "With conclusion ?"
  },
  {
    type: "list",
    choices: CONTENT_TONE,
    name: "tone",
    message: "Content Tone ?",
    default: "Informative"
  },
  {
    type: "number",
    name: "temperature",
    message: "Temperature ?",
    default: 0.8
  },
  {
    type: "number",
    name: "frequencyPenalty",
    message: "Frequency Penalty (-2/2) ?",
    default: 0
  },
  {
    type: "number",
    name: "presencePenalty",
    message: "Presence Penalty (-2/2) ?",
    default: 1
  },
  {
    type: "number",
    name: "logitBias",
    message: "Logit bias ?",
    default: 0
  }
];
var customQuestions = [
  {
    type: "autocomplete",
    name: "language",
    message: "Language ?",
    source: (answersSoFar, input) => LANGUAGES.filter((language) => language.startsWith(input)),
    default: "english"
  },
  {
    type: "list",
    name: "model",
    message: "AI model ?",
    choices: MODELS,
    default: "gpt-4"
  },
  {
    type: "input",
    name: "filename",
    message: "Filename (without extension) ?",
    default: "post"
  },
  {
    type: "input",
    name: "topic",
    message: "Topic/ Artitle Title ? "
  },
  {
    type: "input",
    name: "country",
    message: "Country ?",
    default: "none"
  },
  {
    type: "input",
    name: "intent",
    message: "Intent ?",
    default: "The article should be informative and offer advice to the reader."
  },
  {
    type: "input",
    name: "audience",
    message: "Audience ?",
    default: "The article should be written for a general audience."
  },
  {
    type: "list",
    choices: CONTENT_TONE,
    name: "tone",
    message: "Content Tone ?",
    default: "Informative"
  },
  {
    type: "number",
    name: "temperature",
    message: "Temperature ?",
    default: 0.8
  },
  {
    type: "number",
    name: "frequencyPenalty",
    message: "Frequency Penalty (-2/2) ?",
    default: 0
  },
  {
    type: "number",
    name: "presencePenalty",
    message: "Presence Penalty (-2/2) ?",
    default: 1
  },
  {
    type: "number",
    name: "logitBias",
    message: "Logit bias ?",
    default: 0
  }
];
async function askQuestions() {
  return inquirer.prompt(questions);
}
async function askCustomQuestions() {
  return inquirer.prompt(customQuestions);
}

// src/post.ts
import { oraPromise } from "ora";

// src/lib/post-helpers.ts
import * as dotenv from "dotenv";
import { readFile as rd } from "fs";
import { promisify } from "util";
import { ChatGPTAPI } from "chatgpt";
import pRetry, { AbortError } from "p-retry";

// src/lib/extractor.ts
import JSON5 from "json5";
import { validate } from "jsonschema";
var PostOutlineValidationError = class extends Error {
  constructor(message, errors) {
    super(message);
    this.errors = errors;
  }
};
var schemaValidiation = {
  $schema: "http://json-schema.org/draft-07/schema#",
  type: "object",
  properties: {
    title: {
      type: "string"
    },
    headings: {
      type: "array",
      items: {
        $ref: "#/definitions/Heading"
      }
    },
    slug: {
      type: "string"
    },
    seoTitle: {
      type: "string"
    },
    seoDescription: {
      type: "string"
    }
  },
  required: [
    "title",
    "headings",
    "slug",
    "seoTitle",
    "seoDescription"
  ],
  additionalProperties: false,
  definitions: {
    Heading: {
      type: "object",
      properties: {
        title: {
          type: "string"
        },
        keywords: {
          type: "array",
          items: {
            type: "string"
          }
        },
        headings: {
          type: "array",
          items: {
            $ref: "#/definitions/Heading"
          }
        }
      },
      required: [
        "title"
      ],
      additionalProperties: false
    }
  }
};
function extractCodeBlock(text) {
  const codeBlockTags = ["markdown", "html", "json"];
  for (const tag of codeBlockTags) {
    const regex = new RegExp(`\`\`\`${tag}((.|\\n|\\r)*?)\`\`\``, "i");
    const match = text.match(regex);
    if (match) {
      return match[1];
    }
  }
  const genericRegex = /```\n?((.|\\n|\\r)*?)```/;
  const genericMatch = text.match(genericRegex);
  if (genericMatch) {
    return genericMatch[1];
  }
  return text;
}
function extractPostOutlineFromCodeBlock(text) {
  const jsonData = JSON5.parse(extractCodeBlock(text));
  const v = validate(jsonData, schemaValidiation);
  if (!v.valid) {
    const errorList = v.errors.map((val) => val.toString());
    throw new PostOutlineValidationError("Invalid json for the post outline", errorList);
  }
  return jsonData;
}
function extractJsonArray(text) {
  return JSON5.parse(extractCodeBlock(text));
}
function extractSeoInfo(text) {
  return JSON5.parse(extractCodeBlock(text));
}

// src/lib/prompts.ts
var STRUCTURE_OUTLINE = 'Generate the blog post outline with the following json format : {"title": "", // Add the post title here "headings" : [ { "title": "", //Add the heading title here "keywords": ["...", "...", "...", "..."], // Add a list of keywords here. they will help to generate the final content of this heading."headings": [ // If necessary, add subheadings here. This is optional{ "title": "", "keywords": ["...", "..."] },{ "title": "", "keywords": ["...", "..."] }, ... ] } ... ],"slug" : "", // Use the main keywords for the slug based on the topic of the post. Do not mention the country.  Max 3 or 4 keywords, without stop words and with text normalization and accent stripping"seoTitle" : "", // not the same as the post tile, max 60 characters, do not mention the country"seoDescription : "" //max 155 characters }';
var INFORMATIVE_INTRO_PROMPT = 'Compose the introduction for this blog post topic, without using phrases such as "In this article,..." to introduce the subject.Instead, explain the context and/or explain the main problem. If possible, give some facts. Do not describe or introduce the content of the differents headings of the outline Do not add a heading. Your responses should be in the markdown format.';
var CAPTIVATING_INTO_PROMPT = `Compose a captivating introduction for this blog post topic, without using phrases such as "In this article,..." to introduce the subject.Instead, focus on creating a hook to capture the reader's attention, setting the tone and style, and seamlessly leading the reader into the main content of the article.Your introduction should entice readers to continue reading the article and learn more about the subject presented. Do not add a heading. Your responses should be in the markdown format.`;
function getAutoSystemPrompt(postPrompt) {
  return "You are a copywriter with a strong expertise in SEO. I need a detailed blog post in " + postPrompt.language + ' about the topic : "' + postPrompt.topic + '".';
}
function getCustomSystemPrompt(postPrompt) {
  return postPrompt.prompts[0] + "\nLanguage : " + postPrompt.language + ". " + (postPrompt.topic ? "Topic : " + postPrompt.topic + ". " : "") + (postPrompt.country && postPrompt.country !== "none" ? "Country : " + postPrompt.country + ". " : "") + (postPrompt.intent ? "Intent : " + postPrompt.intent + ". " : "") + (postPrompt.audience ? "Audience : " + postPrompt.audience + ". " : "") + (postPrompt.tone ? "Tone : " + postPrompt.tone + ". " : "");
}
function getPromptForOutline(postPrompt) {
  const { country, intent, audience } = postPrompt;
  const prompt = STRUCTURE_OUTLINE + "Do not add heading for an introduction, conclusion or to summarize the article." + (country === null || country === "none" ? "" : "Market/country/region :" + country + ".") + (audience === null ? "" : "Audience : " + audience + ".") + (intent === null ? "" : "Content intent : " + intent + ".");
  return prompt;
}
function getPromptForMainKeyword() {
  const prompt = "Give me the most important SEO keyword in a json array in which each item match to a word without the stop words";
  return prompt;
}
function getPromptForIntroduction(postPrompt) {
  return !postPrompt.tone || postPrompt.tone === "informative" ? INFORMATIVE_INTRO_PROMPT : CAPTIVATING_INTO_PROMPT;
}
function getPromptForHeading(tone, title, keywords) {
  return tone === "informative" ? getPromptForInformativeHeading(title, keywords) : getPromptForCaptivatingHeading(title, keywords);
}
function getPromptForConclusion() {
  return 'Write a compelling conclusion for this blog post topic, without using transitional phrases such as "in conclusion," "in summary," "in short", "so", "thus", ... or any other transitional expressionFocus on summarizing the main points of the post, emphasizing the significance of the topic, and leaving the reader with a lasting impression or a thought-provoking final remark.Ensure that your conclusion effectively wraps up the article and reinforces the central message or insights presented in the blog post.Do not add a heading. Your responses should be in the markdown format.';
}
function getSeoSystemPrompt(postPrompt) {
  return "You are a SEO expert and you need to optimise an web page. Language : " + postPrompt.language + ". " + (postPrompt.topic ? "Topic : " + postPrompt.topic + ". " : "") + (postPrompt.country ? "Country : " + postPrompt.country + ". " : "") + (postPrompt.topic ? "Topic : " + postPrompt.topic + ". " : "") + (postPrompt.country && postPrompt.country !== "none" ? "Country : " + postPrompt.country + ". " : "") + (postPrompt.intent ? "Intent : " + postPrompt.intent + ". " : "") + (postPrompt.audience ? "Audience : " + postPrompt.audience + ". " : "") + (postPrompt.tone ? "Tone : " + postPrompt.tone + ". " : "");
}
function getPromptForSeoInfo(postPrompt) {
  return "For a content with the following topic : " + postPrompt.topic + ", with the following intent : " + postPrompt.intent + ", write a slug, a seo title and a seo description for this blog post topic in " + postPrompt.language + '.The seo title should be no more than 60 characters long.the seo description should be no more than 155 characters long.Use the main keywords for the slug based on the topic of the post. Do not mention the country.  Max 3 or 4 keywords, without stop words and with text normalization and accent strippingyour response should be in the json format based on the following structure : {"seoTitle": "", "": "seoDescription": "", "slug": ""}';
}
function getPromptForInformativeHeading(title, keywords) {
  const promptAboutKeywords = keywords ? " based on the following list of keywords : " + keywords.join(", ") + "." : "";
  return 'Write an informative content for the heading (without the heading) : "' + title + '"' + promptAboutKeywords + "Do not start the first sentence with the heading. Instead, start with a sentence that introduces and provides context for the heading.Do not add a conclusion or a summary at the end of your answer. Your response should be in the markdown format.";
}
function getPromptForCaptivatingHeading(title, keywords) {
  const promptAboutKeywords = keywords ? " based on the following list of keywords : " + keywords.join(", ") + "." : "";
  return 'Write an captivating content for the heading (without the heading) : "' + title + '"' + promptAboutKeywords + "Ensure to providing in-depth information and valuable insights.Use clear and concise language, along with relevant examples or anecdotes, to engage the reader and enhance their understanding.Do not start the first sentence with the heading. Instead, start with a sentence that introduces and provides context for the heading.Do not add a conclusion or a summary at the end of your answer. Your response should be in the markdown format.";
}

// src/lib/tokenizer.ts
import { get_encoding } from "@dqbd/tiktoken";
var tokenizer = get_encoding("cl100k_base");
function encode(input) {
  return tokenizer.encode(input);
}

// src/lib/template.ts
function extractPrompts(template) {
  const regex = /{\d+:((?:.|\n)*?)}/g;
  return Array.from(template.matchAll(regex)).map((match) => match[1].trim());
}
function replacePrompt(template, index, content) {
  const regex = new RegExp(`{${index}:((?:.|
)*?)}`);
  return template.replace(regex, content);
}
function replaceAllPrompts(template, contents) {
  let tmpTemplate = replacePrompt(template, 0, "");
  contents.forEach((content, index) => {
    tmpTemplate = replacePrompt(tmpTemplate, index + 1, content);
  });
  return tmpTemplate.trim();
}

// src/lib/post-helpers.ts
import { log } from "console";
dotenv.config();
var readFile = promisify(rd);
var ChatGptHelper = class {
  constructor(postPrompt) {
    this.totalTokens = {
      promptTokens: 0,
      completionTokens: 0,
      total: 0
    };
    this.postPrompt = postPrompt;
  }
  isCustom() {
    var _a;
    return ((_a = this.postPrompt) == null ? void 0 : _a.templateFile) !== void 0;
  }
  getPrompt() {
    return this.postPrompt;
  }
  getTotalTokens() {
    return this.totalTokens;
  }
  async init() {
    if (this.isCustom()) {
      if (this.postPrompt.debug) {
        console.log(`Use template : ${this.postPrompt.templateFile}`);
      }
      this.postPrompt.templateContent = await this.readTemplate();
      this.postPrompt.prompts = extractPrompts(this.postPrompt.templateContent);
    }
    const systemMessage = this.isCustom() ? getCustomSystemPrompt(this.postPrompt) : getAutoSystemPrompt(this.postPrompt);
    await this.buildChatGPTAPI(systemMessage);
  }
  async buildChatGPTAPI(systemMessage) {
    var _a;
    this.api = new ChatGPTAPI({
      apiKey: ((_a = this.postPrompt) == null ? void 0 : _a.apiKey) || process.env.OPENAI_API_KEY,
      completionParams: {
        model: this.postPrompt.model
      },
      systemMessage,
      debug: this.postPrompt.debugapi
    });
    if (this.postPrompt.debug) {
      console.log(`OpenAI API initialized with model : ${this.postPrompt.model}`);
    }
    this.completionParams = {
      temperature: this.postPrompt.temperature ?? 0.8,
      frequency_penalty: this.postPrompt.frequencyPenalty ?? 0,
      presence_penalty: this.postPrompt.presencePenalty ?? 1
    };
    if (this.postPrompt.logitBias) {
      const mainKwWords = await this.generateMainKeyword();
      const logitBiais = {};
      mainKwWords.forEach((kw) => {
        const encoded = encode(kw);
        encoded.forEach((element) => {
          logitBiais[element] = Number(this.postPrompt.logitBias) || -1;
        });
      });
      this.completionParams.logit_bias = logitBiais;
    }
    if (this.postPrompt.debug) {
      console.log("---------- COMPLETION PARAMETERS ----------");
      console.log("Max Tokens  : " + this.completionParams.max_tokens);
      console.log("Temperature : " + this.completionParams.temperature);
      console.log("Frequency Penalty : " + this.completionParams.frequency_penalty);
      console.log("Presence Penalty : " + this.completionParams.presence_penalty);
      console.log("Logit Biais : " + this.completionParams.logit_bias);
    }
  }
  async generateMainKeyword() {
    const prompt = getPromptForMainKeyword();
    if (this.postPrompt.debug) {
      console.log("---------- PROMPT MAIN KEYWORD ----------");
      console.log(prompt);
    }
    const response = await this.sendRequest(prompt);
    if (this.postPrompt.debug) {
      console.log("---------- MAIN KEYWORD ----------");
      console.log(response.text);
    }
    return extractJsonArray(response.text);
  }
  async generateContentOutline() {
    const prompt = getPromptForOutline(this.postPrompt);
    if (this.postPrompt.debug) {
      console.log("---------- PROMPT OUTLINE ----------");
      console.log(prompt);
    }
    this.chatParentMessage = await this.sendRequest(prompt);
    if (this.postPrompt.debug) {
      console.log("---------- OUTLINE ----------");
      console.log(this.chatParentMessage.text);
    }
    return extractPostOutlineFromCodeBlock(this.chatParentMessage.text);
  }
  async generateIntroduction() {
    const response = await this.sendRequest(getPromptForIntroduction(this.postPrompt), this.completionParams);
    return extractCodeBlock(response.text);
  }
  async generateConclusion() {
    const response = await this.sendRequest(getPromptForConclusion(), this.completionParams);
    return extractCodeBlock(response.text);
  }
  async generateHeadingContents(postOutline) {
    const headingLevel = 2;
    return await this.buildContent(postOutline.headings, headingLevel);
  }
  async buildContent(headings, headingLevel, previousContent = "") {
    if (headings.length === 0) {
      return previousContent;
    }
    const [currentHeading, ...remainingHeadings] = headings;
    const mdHeading = Array(headingLevel).fill("#").join("");
    let content = previousContent + "\n" + mdHeading + " " + currentHeading.title;
    if (currentHeading.headings && currentHeading.headings.length > 0) {
      content = await this.buildContent(currentHeading.headings, headingLevel + 1, content);
    } else {
      content += "\n" + await this.getContent(currentHeading);
    }
    return this.buildContent(remainingHeadings, headingLevel, content);
  }
  async getContent(heading) {
    if (this.postPrompt.debug) {
      console.log(`
Heading : ${heading.title}  ...'
`);
    }
    const response = await this.sendRequest(getPromptForHeading(this.postPrompt.tone, heading.title, heading.keywords), this.completionParams);
    return `${extractCodeBlock(response.text)}
`;
  }
  async generateCustomPrompt(customPrompt) {
    this.chatParentMessage = await this.sendRequest(customPrompt, this.completionParams);
    return extractCodeBlock(this.chatParentMessage.text);
  }
  async generateSeoInfo() {
    const systemPrompt = getSeoSystemPrompt(this.postPrompt);
    await this.buildChatGPTAPI(systemPrompt);
    this.chatParentMessage = await this.sendRequest(getPromptForSeoInfo(this.postPrompt), this.completionParams);
    log("---------- SEO INFO ----------");
    console.log(this.chatParentMessage.text);
    return extractSeoInfo(this.chatParentMessage.text);
  }
  async sendRequest(prompt, completionParams) {
    return await pRetry(async () => {
      var _a;
      const options = { parentMessageId: (_a = this.chatParentMessage) == null ? void 0 : _a.id };
      if (completionParams) {
        options.completionParams = completionParams;
      }
      const response = await this.api.sendMessage(prompt, options);
      this.totalTokens.promptTokens += response.detail.usage.prompt_tokens;
      this.totalTokens.completionTokens += response.detail.usage.completion_tokens;
      this.totalTokens.total += response.detail.usage.total_tokens;
      return response;
    }, {
      retries: 10,
      onFailedAttempt: async (error) => {
        if (this.postPrompt.debug) {
          console.log("---------- OPENAI REQUEST ERROR ----------");
          console.log(error);
        }
        if (error instanceof AbortError) {
          console.log("OpenAI API - Request aborted");
        } else {
          console.log(`OpenAI API - Request failed - Attempt ${error.attemptNumber} failed. There are ${error.retriesLeft} retries left.`);
        }
      }
    });
  }
  async readTemplate() {
    const templatePath = this.postPrompt.templateFile;
    return await readFile(templatePath, "utf-8");
  }
};

// src/post.ts
var PostGenerator = class {
  constructor(helper) {
    this.helper = helper;
  }
  async generate() {
    return this.helper.isCustom() ? await this.customGenerate() : await this.autoGenerate();
  }
  async customGenerate() {
    const promptContents = [];
    await oraPromise(
      this.helper.init(),
      {
        text: " Init the completion parameters ..."
      }
    );
    this.helper.getPrompt().prompts.shift();
    for (const prompt of this.helper.getPrompt().prompts) {
      const content2 = await oraPromise(
        this.helper.generateCustomPrompt(prompt),
        {
          text: "Generating post prompt ..."
        }
      );
      promptContents.push(content2);
    }
    const content = replaceAllPrompts(this.helper.getPrompt().templateContent, promptContents);
    const seoInfo = await oraPromise(
      this.helper.generateSeoInfo(),
      {
        text: "Generating SEO info ..."
      }
    );
    return {
      title: this.helper.getPrompt().topic,
      slug: seoInfo.slug,
      seoTitle: seoInfo.seoTitle,
      seoDescription: seoInfo.seoDescription,
      content,
      totalTokens: this.helper.getTotalTokens()
    };
  }
  async autoGenerate() {
    await oraPromise(
      this.helper.init(),
      {
        text: " Init the completion parameters ..."
      }
    );
    const tableOfContent = await oraPromise(
      this.helper.generateContentOutline(),
      {
        text: "Generating post outline ..."
      }
    );
    let content = await oraPromise(
      this.helper.generateIntroduction(),
      {
        text: "Generating introduction..."
      }
    );
    content += await oraPromise(
      this.helper.generateHeadingContents(tableOfContent),
      {
        text: "Generating content ..."
      }
    );
    if (this.helper.getPrompt().withConclusion) {
      content += await oraPromise(
        this.helper.generateConclusion(),
        {
          text: "Generating conclusion..."
        }
      );
    }
    return {
      title: tableOfContent.title,
      slug: tableOfContent.slug,
      seoTitle: tableOfContent.seoTitle,
      seoDescription: tableOfContent.seoDescription,
      content,
      totalTokens: this.helper.getTotalTokens()
    };
  }
};
var OpenAIPostGenerator = class extends PostGenerator {
  constructor(postPrompt) {
    super(new ChatGptHelper(postPrompt));
  }
};

// src/bin/command/post.ts
var GPT4_PROMPT_PRICE = 0.03;
var GPT4_COMPLETION_PRICE = 0.06;
var GPT35_PROMPT_PRICE = 2e-3;
var GPT_COMPLETION_PRICE = 2e-3;
function buildPostCommands(program) {
  program.command("post").description("Generate a post").option("-t, --templateFile <file>", "set the template file (optional)").option("-d, --debug", "output extra debugging").option("-da, --debugapi", "debug the api calls").option("-k, --apiKey <key>", "set the OpenAI api key (optional, you can also set the OPENAI_API_KEY environment variable)").action(async (options) => {
    await generatePost(options);
  });
}
async function generatePost(options) {
  const isCustomMode = isCustom(options);
  const answers = isCustomMode ? await askCustomQuestions() : await askQuestions();
  const postPrompt = {
    ...options,
    ...answers,
    maxModelTokens: answers.model === "gpt-4" ? 8e3 : 4e3
  };
  const postGenerator = new OpenAIPostGenerator(postPrompt);
  const post2 = await postGenerator.generate();
  const jsonData = {
    ...post2,
    // In custom mode, we don't want to render the content
    // We keep the content as it is in the template
    content: isCustomMode ? post2.content : marked(post2.content)
  };
  const writeJSONPromise = fs.promises.writeFile(`${answers.filename}.json`, JSON.stringify(jsonData), "utf8");
  const writeDocPromise = fs.promises.writeFile(`${answers.filename}.${getFileExtension(options)}`, buildContent(options, post2), "utf8");
  await Promise.all([writeJSONPromise, writeDocPromise]);
  console.log(`\u{1F525} Content is created successfully in ${answers.filename}.*`);
  console.log(`- Slug : ${post2.slug}`);
  console.log(`- SEO Title : ${post2.seoTitle}`);
  console.log(`- SEO Description : ${post2.seoDescription}`);
  console.log(`- Total prompts tokens : ${post2.totalTokens.promptTokens}`);
  console.log(`- Total completion tokens : ${post2.totalTokens.completionTokens}`);
  console.log(`- Estimated cost :  ${estimatedCost(postPrompt.model, post2)}$`);
}
function buildContent(options, post2) {
  return isCustom(options) ? post2.content : "# " + post2.title + "\n" + post2.content;
}
function estimatedCost(model, post2) {
  const promptTokens = post2.totalTokens.promptTokens;
  const completionTokens = post2.totalTokens.completionTokens;
  return model === "gpt-4" ? Number(promptTokens / 1e3 * GPT4_PROMPT_PRICE + completionTokens / 1e3 * GPT4_COMPLETION_PRICE).toFixed(4) : Number(promptTokens / 1e3 * GPT35_PROMPT_PRICE + completionTokens / 1e3 * GPT_COMPLETION_PRICE).toFixed(4);
}
function isCustom(options) {
  return options.templateFile !== void 0;
}
function getFileExtension(options) {
  return isCustom(options) ? options.templateFile.split(".").pop() : "md";
}

// src/bin/command/wp.ts
import fs3 from "fs";
import util2 from "util";

// src/lib/store/store.ts
import os from "os";
import fs2 from "fs";
import path from "path";
import util from "util";
var readFile2 = util.promisify(fs2.readFile);
var userHomeDir = os.homedir();
var HIDDEN_DIR_NAME = ".julius";
var hiddenDirPath = path.join(userHomeDir, HIDDEN_DIR_NAME);
var WORDPRESS_FILE = `${hiddenDirPath}/wordpress.json`;
function initStore() {
  if (!fs2.existsSync(hiddenDirPath)) {
    fs2.mkdirSync(hiddenDirPath);
  }
  if (!fs2.existsSync(WORDPRESS_FILE)) {
    fs2.writeFileSync(WORDPRESS_FILE, "[]", "utf8");
  }
}
async function getAllWordpress() {
  const data = await readFile2(WORDPRESS_FILE, "utf8");
  return JSON.parse(data);
}
async function addWordpress(wp) {
  const wpSites = [...await getAllWordpress(), wp].sort((a, b) => a.domain.localeCompare(b.domain));
  await fs2.promises.writeFile(WORDPRESS_FILE, JSON.stringify(wpSites), "utf8");
}
async function getWordpress(domain) {
  const wpSites = await getAllWordpress();
  const index = !isNaN(Number(domain)) ? Number(domain) - 1 : wpSites.findIndex((wp) => wp.domain === domain);
  return wpSites[index];
}
async function removeWordpress(domain) {
  const wpSites = await getAllWordpress();
  const index = !isNaN(Number(domain)) ? Number(domain) - 1 : wpSites.findIndex((wp) => wp.domain === domain);
  if (index < 0) {
    return false;
  }
  wpSites.splice(index, 1);
  await fs2.promises.writeFile(WORDPRESS_FILE, JSON.stringify(wpSites), "utf8");
  return true;
}
async function exportWordpressList(exportFile) {
  await fs2.promises.copyFile(WORDPRESS_FILE, exportFile);
}
async function importWordpressList(importFile) {
  const data = await readFile2(importFile, "utf8");
  const wpSites = JSON.parse(data);
  await fs2.promises.writeFile(WORDPRESS_FILE, JSON.stringify(wpSites), "utf8");
}

// src/lib/wp/wp-api.ts
import axios from "axios";
async function getCategories(wp) {
  const { domain, username, password } = wp;
  const response = await axios.get(`${getApiUrl(domain)}/categories`, authenticate(username, password));
  return response.data.map((category) => {
    return {
      id: category.id,
      name: category.name,
      slug: category.slug
    };
  });
}
async function post(wp, post2, isYoastPlugin) {
  const { domain, username, password } = wp;
  const postData = {
    ...post2
  };
  if (isYoastPlugin) {
    postData.yoast_meta = {
      yoast_wpseo_title: post2.seoTitle,
      yoast_wpseo_metadesc: post2.seoDescription
    };
  }
  return await axios.post(`${getApiUrl(domain)}/posts`, postData, authenticate(username, password));
}
function getApiUrl(domain) {
  return `https://${domain}/wp-json/wp/v2`;
}
function authenticate(username, password) {
  const token = Buffer.from(`${username}:${password}`, "utf8").toString("base64");
  return {
    headers: {
      Authorization: `Basic ${token}`,
      "Content-Type": "application/json"
    }
  };
}

// src/bin/command/wp.ts
var readFile3 = util2.promisify(fs3.readFile);
function buildWpCommands(program) {
  const wpCommand = program.command("wp").description("Wordpress related commands. The wp list is stored in the local store : ~/.julius/wordpress.json").action(() => {
    console.log('Please provide a sub-command: "add" or "ls" or "rm" , ... ');
  });
  wpCommand.command("ls").description("List all Wordpress sites").action(async () => {
    await getAllWp();
  });
  wpCommand.command("info <domain|index>").description("Info on a Wordpress site").action(async (domain) => {
    const domainFound = await getWordpress(domain);
    if (domainFound) {
      console.log("\nWordpress site found :\n");
      console.log(`
domain : ${domainFound.domain}`);
      console.log(`username : ${domainFound.username}`);
      console.log(`password : ${domainFound.password}
`);
    } else {
      console.log(`
Wordpress site ${domain} not found
`);
    }
  });
  wpCommand.command("add <domain:username:password>").description("Add a new Wordpress site").action(async (site) => {
    await addWpSite(site);
  });
  wpCommand.command("rm <domain|index>").description("Remove Wordpress site").action(async (domain) => {
    const deleted = await removeWordpress(domain);
    console.log(deleted ? `
Wordpress site ${domain} removed successfully
` : `Wordpress site ${domain} not found
`);
  });
  wpCommand.command("export <file>").description("Export the list of wordpress sites (with credentials) the console").action(async (file) => {
    await exportWordpressList(file);
  });
  wpCommand.command("import <file>").description("Import the list of wordpress sites (with credentials) from a file").action(async (file) => {
    await importWordpressList(file);
  });
  wpCommand.command("categories <domain|index>").description("Fetch the categories for one Wordpress site").action(async (domain) => {
    const domainFound = await getWordpress(domain);
    if (domainFound) {
      const categories = await getCategories(domainFound);
      console.log(categories);
    } else {
      console.log(`
Wordpress site ${domain} not found
`);
    }
  });
  wpCommand.command("post <domain> <categoryId> <hasYoastPlugin> <jsonfile>").description("Post a new article to a Wordpress site. The file has to be a json file containing : { content, categories, seoTitle, seoDescription }").action(async (domain, categoryId, hasYoastPlugin, jsonFile) => {
    const domainFound = await getWordpress(domain);
    if (!domainFound) {
      console.log(`
Wordpress site ${domain} not found
`);
      return;
    }
    const jsonContent = await readFile3(jsonFile, "utf8");
    const post2 = JSON.parse(jsonContent);
    post2.categories = [categoryId];
    post2.status = "draft";
    await post(domainFound, post2, hasYoastPlugin === "true");
    console.log(`
Content has been published on https://${domainFound.domain}/${post2.slug}
`);
  });
}
async function getAllWp() {
  const wpSites = await getAllWordpress();
  if (wpSites.length === 0) {
    console.log("\nNo Wordpress site found\n");
    return;
  }
  console.log("\nWordpress sites :\n");
  console.log(wpSites.map((wp, index) => `${index + 1}. ${wp.domain} (${wp.username})`).join("\n") + "\n");
}
async function addWpSite(site) {
  const [domain, username, password] = site.split(":");
  if (!domain || !username || !password) {
    console.error("Invalid format for adding a new wp site. Expected : domain:username:password");
    return;
  }
  await addWordpress({ domain, username, password });
  console.log(`
Wordpress site ${domain} added successfully
`);
}
export {
  OpenAIPostGenerator,
  PostGenerator,
  addWordpress,
  buildPostCommands,
  buildWpCommands,
  exportWordpressList,
  getAllWordpress,
  getWordpress,
  importWordpressList,
  initStore,
  removeWordpress
};
//# sourceMappingURL=index.js.map